"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Endpoint = void 0;
const axios_1 = require("axios");
const lisk_sdk_1 = require("lisk-sdk");
const schemas_1 = require("./schemas");
const validator = lisk_sdk_1.validator.validator;
class Endpoint extends lisk_sdk_1.BasePluginEndpoint {
    constructor() {
        super(...arguments);
        this._state = { publicKey: undefined, passphrase: undefined };
    }
    init(state, apiClient, config) {
        this._state = state;
        this._client = apiClient;
        this._config = config;
    }
    async authorize(context) {
        validator.validate(schemas_1.authorizeParamsSchema, context.params);
        const { enable, password } = context.params;
        try {
            const parsedEncryptedPassphrase = lisk_sdk_1.cryptography.encrypt.parseEncryptedMessage(this._config.encryptedPassphrase);
            const passphrase = await lisk_sdk_1.cryptography.encrypt.decryptMessageWithPassword(parsedEncryptedPassphrase, password, 'utf-8');
            const { publicKey } = lisk_sdk_1.cryptography.legacy.getPrivateAndPublicKeyFromPassphrase(passphrase);
            this._state.publicKey = enable ? publicKey : undefined;
            this._state.passphrase = enable ? passphrase : undefined;
            const changedState = enable ? 'enabled' : 'disabled';
            return {
                result: `Successfully ${changedState} the faucet.`,
            };
        }
        catch (error) {
            throw new Error('Password given is not valid.');
        }
    }
    async fundTokens(context) {
        var _a;
        validator.validate(schemas_1.fundParamsSchema, context.params);
        const { address, token } = context.params;
        if (!this._state.publicKey || !this._state.passphrase) {
            throw new Error('Faucet is not enabled.');
        }
        const captchaResult = await (0, axios_1.default)({
            method: 'post',
            url: 'https://www.google.com/recaptcha/api/siteverify',
            params: {
                secret: this._config.captchaSecretkey,
                response: token,
            },
        });
        if (!((_a = captchaResult === null || captchaResult === void 0 ? void 0 : captchaResult.data) === null || _a === void 0 ? void 0 : _a.success)) {
            throw new Error('Captcha response was invalid.');
        }
        await this._transferFunds(address);
        return {
            result: `Successfully funded account at address: ${address}.`,
        };
    }
    async _transferFunds(address) {
        var _a;
        const transferTransactionParams = {
            amount: lisk_sdk_1.transactions.convertLSKToBeddows(this._config.amount),
            recipientAddress: address,
            data: '',
        };
        const transaction = await this._client.transaction.create({
            module: 'token',
            command: 'transfer',
            senderPublicKey: (_a = this._state.publicKey) === null || _a === void 0 ? void 0 : _a.toString('hex'),
            fee: lisk_sdk_1.transactions.convertLSKToBeddows(this._config.fee),
            params: transferTransactionParams,
        }, this._state.passphrase);
        await this._client.transaction.send(transaction);
    }
}
exports.Endpoint = Endpoint;
//# sourceMappingURL=endpoint.js.map